#!/bin/sh
extractrc ../src/*.rc >> ../src/rc.cpp
xgettext -C -j -ki18n -kI18N_NOOP ../src/*.cpp -o pikdev.pot
xgettext -C -j -ki18n -kI18N_NOOP ../src/*.cpp -o es.po
xgettext -C -j -ki18n -kI18N_NOOP ../src/*.cpp -o fr.po
xgettext -C -j -ki18n -kI18N_NOOP ../src/*.cpp -o de.po
xgettext -C -j -ki18n -kI18N_NOOP ../src/*.cpp -o it.po
