#include "TestWindow.h"

#include <iostream>
#include <qapplication.h>
#include <kpopupmenu.h>
#include <kmenubar.h>
#include <kfiledialog.h>
#include <kurl.h>
#include <klocale.h>
#include <qlistview.h>
#include <qstringlist.h>
#include <kconfig.h>

using namespace std ;

TestWindow ::TestWindow(QWidget *w)  :  KMainWindow(w)
{
  
  show() ;

}

TestWindow ::~TestWindow()
{
}

