<!DOCTYPE kpartgui>
<kpartgui version="1" name="pikdev" >
<MenuBar noMerge="1" >
<Menu name="file" >
<text>&amp;File</text>
   <Action name="new" />
   <Action name="open" />
   <Action name="open_recent_file" />
   <Separator/>
   <Action name="save" />
   <Action name="save_as" />
   <Action name="save_all" />
   <Separator/>
   <Action name="close" />
   <Action name="close_all" />
   <Separator/>
  <Action name="print_dialog" />
     <Separator/>
   <Action name="quit" />  </Menu>
  <Menu name="edit" >
<text>&amp;Edit</text>
   <Action name="undo" />
   <Action name="redo" />
   <Action name="cut" />
   <Action name="copy" />
   <Action name="paste" />
   <Separator/>
   <Action name="find" />
   <Action name="find_next" />
   <Action name="find_previous" />
   <Separator/>
   <Action name="replace" />
   <Separator/>
   <Action name="goto_line" />
   <Separator/>
   <Action name="comment" />
   <Action name="uncomment" />  
   <Separator/>
   <Action name="indent" />
   </Menu>
  <Menu name="display" >
<text>&amp;Display</text>
   <Action name="split_view" />
   <Action name="remove_view" />
   <Separator/>
   <Action name="line_numbers" />
   <Action name="icon_borders" />
   <Action name="auto_wrap" />
   <Action name="dos_eol" />

   </Menu>
   
  <Action name="bookmarks" />
  
  <Action name="windows" />

  <Menu name="project" >
<text>&amp;Project</text>
   <Action name="new_project" />
   <Action name="open_project" />
   <Action name="open_recent_project" />
   <Separator/>
   <Action name="edit_project" />
   <Separator/>
   <Action name="close_project" />
   <Separator/>
   <Action name="add_to_project" />
   <Action name="addcurrent_to_project" />
 </Menu>
 
  <Menu name="tools" >
<text>&amp;Tools</text>
   <Action name="assemble" />
   <Action name="deassemble" />
   <Action name="program" />
   <Separator/>
   <Action name="pikloops" />
  </Menu>
   
  <Menu name="configure" >
<text>&amp;Configure</text>
   <Action name="configure-center" />
   <Action name="configuretoolbar" />
   <Action name="configure-editor" />  </Menu>
  <Menu name="help" >
<text>&amp;Help</text>
   <Action name="help_contents" />
   <Action name="help_about_app" />
   <Action name="help_about_kde" />
</Menu>
  <ToolBar noMerge="1" name="fileToolBar" >
<text>File Toolbar</text>
   <Action group="file" name="new" />
   <Action group="file" name="open" />
   <Action group="file" name="save" />
   <Action group="file" name="save_as" />
   <Action group="file" name="save_all" />
   <Separator name="separator_1" />
   <Action group="file" name="print_dialog" />
   <Separator name="separator_1" />
   <Action group="file" name="close" />
   <Separator name="separator_0" />
   <Action name="quit" />
   </ToolBar>

   
  <ToolBar noMerge="1" name="editToolBar" >
<text>Edit Toolbar</text>
   <Action group="edit" name="undo" />
   <Action group="edit" name="redo" />
   <Action group="edit" name="cut" />
   <Action group="edit" name="copy" />
   <Action group="edit" name="paste" />
   <Action group="edit" name="find" />
   <Action group="edit" name="goto_line" />  </ToolBar>
   
  <ToolBar noMerge="1" name="projectToolBar" >
<text>Project Toolbar</text>
   <Action group="project" name="new_project" />
   <Action group="project" name="open_project" />
   <Separator name="separator_1" />
   <Action group="project" name="edit_project" />
   <Action group="project" name="close_project" />
   <Separator name="separator_1" />
   <Action group="project" name="add_to_project" />
   <Action group="project" name="addcurrent_to_project" />
   </ToolBar>
   
  <ToolBar noMerge="1" name="toolsToolBar" >   
<text>Tools Toolbar</text>
   <Action group="tools" name="assemble" />
   <Action group="tools" name="deassemble" />
   <Action group="tools" name="program" />
   </ToolBar>

   
   </MenuBar>
   </kpartgui>