#ifndef TESTWINDOW_H
#define TESTWINDOW_H
#include <kparts/mainwindow.h>
#include <kparts/event.h>
#include <kate/view.h>
#include <kate/document.h>
#include <klibloader.h>
#include <kstdaction.h>
 
#include "KBEditor.h"

class TestWindow : public  KMainWindow
{
Q_OBJECT
private:

public:
	TestWindow(QWidget *w);
	virtual ~TestWindow() ;		 
 public slots:
} ;


#endif
