/***************************************************************************
                          NewProject.h  -  description
                             -------------------
    begin                : ven nov 28 2003
    copyright            : (C) 2003 by Alain Gibaud
    email                : alain.gibaud@univ-valenciennes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef NEWPROJECT_H
#define NEWPROJECT_H

#include <qwidget.h>
#include <qmainwindow.h>

class QLineEdit ;
class QTextEdit ;
class QString ;
class QComboBox ;
class QCheckBox ;

/**Widget for specifying informations about a new project
  *@author Alain Gibaud
  */

class NewProject : public QMainWindow
{
   Q_OBJECT
public: 
	NewProject();
	~NewProject();
  /** No descriptions */
  bool createProjectEntry();
public: // Public attributes
  /**  */
private:
    QLineEdit *project_name,
      *project_dir  ;
    QComboBox *project_type ;
    QCheckBox *config_now ;

private slots: // Private slots
  /** This slot must be activated from the browse button and update the project_dir string. */
  void slotSelectDir();
//  void slotSelectOut() ;
  void slotCancel();
  /** Collect project data and emit the projectData signal */
  void slotCreate();
  /** No descriptions */
  void slotNameChanged(const QString &);
signals: // Signals
  /** No descriptions */
  void projectData(const QString& name, const QString& dir, const QString& type) ;
};

#endif
