/***************************************************************************
                          ConfigCenter.h  -  description
                             -------------------
    begin                : mer oct 13 2004
    copyright            : (C) 2004 by Gibaud Alain
    email                : alain.gibaud@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONFIGCENTER_H
#define CONFIGCENTER_H

#include <qwidget.h>
#include <qmainwindow.h>

/**Tabbed organizer for ConfigWidgets

  *@author Gibaud Alain
  */


class QTabWidget ;
class QPushButton ;
class ConfigWidget ;
class QToolBox ;
class QListView ;
class QWidgetStack ;
class QListViewItem;
class QLabel ;
class QSplitter ;
class PikMain ;
  
class ConfigCenter : public QMainWindow
{
   Q_OBJECT
public: 
	ConfigCenter(PikMain *main);
	~ConfigCenter();
  /** Insert a new config widget */
  void plugConfigWidget(ConfigWidget *c);
private: // Private attributes
  /**  */
  QLabel *title , *image;
  /**  */
  QPushButton * applyButton;
  /**  */
  QPushButton * cancelButton;
  /**  */
  QPushButton * okButton;
  /**  */
  QListView * confView;
  /** Configurator widgets stack */
  QWidgetStack * wstack;
  /**  */
  QSplitter *splitv ;
public slots: // Public slots
  /** No descriptions */
  virtual void slotOk();
  /** No descriptions */
  virtual void slotClose();
  /** No descriptions */
  virtual void slotApply();
public slots: // Public slots
  /** No descriptions */
  void raiseConfigWidget(QListViewItem *item);
};

#endif
