<!DOCTYPE kpartgui>
<kpartgui name="pikdev" version="1">

<MenuBar noMerge="1">
  <Menu name="file"><text>&amp;File</text>
    <Action name="loadhexas" />
    <Action name="loadhex" />
    <Action name="savehex" />
    <Action name="savehexas"/>
    <Separator/>
    <Action name="quitprog"/>
  </Menu>

  <Menu name="ops"><text>&amp;PIC Operations</text>
    <Action name="readchip" />
    <Action name="erasechip" />
    <Action name="burnchip" />
    <Action name="verifychip"/>
  </Menu>

  <Action name="extra-ops" />
  
  <Menu name="config"><text>&amp;Configure</text>
    <Action name="configuretoolbar" />
  </Menu>

<ToolBar name="mainToolBar" noMerge="1"><text>File Toolbar</text>

  <Action name="loadhexas" group="filehex" />
  <Action name="savehex" group="filehex" />
  <Action name="savehexas" group="filehex" />
  <Action name="loadhex" group="filehex" />

  <Separator/>
  
  <Action name="quitprog" group="filehex" />  
</ToolBar>

<ToolBar name="burnToolBar" noMerge="1"><text>Chip Toolbar</text>
  <Action name="readchip" group="chip" />
  <Action name="erasechip" group="chip" />
  <Action name="burnchip" group="chip" />
  <Action name="verifychip" group="chip" />
  <Separator/>
</ToolBar>

</kpartgui>
