#include "XQPushButton.h"

XQPushButton::XQPushButton(const QString& text, QWidget *parent, int identifier, const char *name)
 : QPushButton(text, parent, name), ident(identifier)
{
	connect(this, SIGNAL(clicked()), this , SLOT(slotClicked())) ;
}


XQPushButton::~XQPushButton()
{
}

int XQPushButton::getID()
{
	return ident ;
}
// #include "XQPushButton.moc"

/*!
    \fn XQPushButton::slotClicked()
 */
void XQPushButton::slotClicked()
{
    emit(clicked(ident));
}
