/***************************************************************************
                          ConfigWidget.h  -  description
                             -------------------
    begin                : mer oct 13 2004
    copyright            : (C) 2004 by Gibaud Alain
    email                : alain.gibaud@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONFIGWIDGET_H
#define CONFIGWIDGET_H

#include <qframe.h>
#include <qvbox.h>
#include <qstring.h>
#include <qpixmap.h>

/**Base class for configuration widgets
  *@author Gibaud Alain
  */

class ConfigWidget : public QVBox  {
   Q_OBJECT
public: 
	ConfigWidget(QWidget *parent=0, const char *name=0);
	virtual ~ConfigWidget();
public slots: // Public slots
   /** No descriptions */
  virtual void doApply() = 0  ;
  /** Return the name of the config widget
 */
  virtual QString configName() = 0;
  /** Return the title of the config page
 */
  virtual QString configTitle() = 0;
  /** No descriptions */
  virtual QPixmap icon() = 0;
};

#endif
