/***************************************************************************
                          BinWordsEditor.h  -  description
                             -------------------
    begin                : jeu jun 26 2003
    copyright            : (C) 2003 by Alain Gibaud
    email                : alain.gibaud@univ-valenciennes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BINWORDSEDITOR_H
#define BINWORDSEDITOR_H

#include <qvgroupbox.h>
#include <qwidgetstack.h>

#include "FragBuffer.hh"
#include "BinWordEditor.h"
#include <vector>
using std::vector ;
/**Multi Words binary editor for config registers
  *@author Alain Gibaud
  */

class BinWordsEditor : public QVGroupBox
{
Q_OBJECT
  vector<BinWordEditor *> weditors ;
  QWidgetStack *stack ;
public: 
	BinWordsEditor(FragBuffer& me, unsigned int index1 , unsigned int index2,
	    const QString& desc,
	    const QString& title,bool byteAddr, QWidget *parent);
	virtual ~BinWordsEditor();
public slots: // Public slots
  /** raise a word editor widget */
  void slotRaiseBWE(int numwidget);
signals:
  void wordsChanged() ;
};

#endif
