/***************************************************************************
                          popupwindow.h  -  description
                             -------------------
    begin                : lun mar 15 2004
    copyright            : (C) 2004 by Alain Gibaud
    email                : alain.gibaud@univ-valenciennes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef POPUPWINDOW_H
#define POPUPWINDOW_H

#include <qwidget.h>
#include <kpassivepopup.h>

/**A yellow popup window for temporarily displaying text information
  *@author Alain Gibaud
  */

class PopupWindow : public KPassivePopup  {
   Q_OBJECT
public: 
	PopupWindow(QWidget *parent=0, const QString& txt="");
	~PopupWindow();
};

#endif
